@@ -40,7 +40,14 @@ from solo import helpers
     hidden=True,
     help="Development option: pull firmware from http://localhost:8000",
 )
-def update(serial, yes, hacker, secure, local_firmware_server):
+@click.option(
+    "--alpha",
+    is_flag=True,
+    default=False,
+    hidden=True,
+    help="Development option: use release refered to by ALPHA_VERSION",
+)
+def update(serial, yes, hacker, secure, local_firmware_server, alpha):
     """Update Solo key to latest firmware version."""
 
     # Check exactly one of --hacker/--secure is selected
@@ -100,24 +107,38 @@ def update(serial, yes, hacker, secure, local_firmware_server):
 
     # Get firmware version to use
     try:
-        r = requests.get(
-            "https://raw.githubusercontent.com/solokeys/solo/master/STABLE_VERSION"
+        if alpha:
+            version_file = "ALPHA_VERSION"
+        else:
+            version_file = "STABLE_VERSION"
+        fetch_url = (
+            f"https://raw.githubusercontent.com/solokeys/solo/master/{version_file}"
         )
+
+        r = requests.get(fetch_url)
         if r.status_code != 200:
-            print("Could not fetch stable version name from solokeys/solo repository!")
+            print(
+                f"Could not fetch version name from {version_file} in solokeys/solo repository!"
+            )
             sys.exit(1)
+
         version = r.text.split()[0].strip()
         # Windows BOM haha
         # if version.encode() == b'\xef\xbf\xbd\xef\xbf\xbd1\x00.\x001\x00.\x000\x00':
         #     version = '1.1.0'
         try:
             assert version.count(".") == 2
-            major, minor, patch = map(int, version.split("."))
+            major, minor, patch_and_more = version.split(".")
+            if "-" in patch_and_more:
+                patch, pre = patch_and_more.split("-")  # noqa: F841
+            else:
+                patch, pre = patch_and_more, None  # noqa: F841
+            major, minor, patch = map(int, (major, minor, patch))
         except Exception:
             print(f"Abnormal version format '{version}'")
             sys.exit(1)
     except Exception:
-        print("Error fetching stable version name from solokeys/solo repository!")
+        print("Error fetching version name from solokeys/solo repository!")
         sys.exit(1)
 
     # Get firmware to use
@@ -127,9 +148,10 @@ def update(serial, yes, hacker, secure, local_firmware_server):
         base_url = f"https://github.com/solokeys/solo/releases/download/{version}"
 
     if hacker:
-        firmware_url = f"{base_url}/firmware-hacker-{version}.hex"
+        firmware_file_github = f"firmware-hacker-{version}.hex"
     else:
-        firmware_url = f"{base_url}/firmware-secure-{version}.json"
+        firmware_file_github = f"firmware-secure-{version}.json"
+    firmware_url = f"{base_url}/{firmware_file_github}"
 
     extension = firmware_url.rsplit(".")[-1]
 
@@ -137,7 +159,7 @@ def update(serial, yes, hacker, secure, local_firmware_server):
         r = requests.get(firmware_url)
         if r.status_code != 200:
             print(
-                "Could not official firmware build from solokeys/solo repository releases!"
+                "Could not fetch official firmware build from solokeys/solo repository releases!"
             )
             print(f"URL attempted: {firmware_url}")
             sys.exit(1)
@@ -148,15 +170,12 @@ def update(serial, yes, hacker, secure, local_firmware_server):
                 json_content = json.loads(content.decode())
             except Exception:
                 print(f"Invalid JSON content fetched from {firmware_url}!")
-                import IPython
-
-                IPython.embed()
                 sys.exit(1)
 
         with tempfile.NamedTemporaryFile(suffix="." + extension, delete=False) as fh:
             fh.write(r.content)
             firmware_file = fh.name
-            print(f"Wrote temporary copy to {firmware_file}")
+            print(f"Wrote temporary copy of {firmware_file_github} to {firmware_file}")
     except Exception:
         print("Problem fetching {firmware_url}!")
         sys.exit(1)
@@ -170,8 +189,6 @@ def update(serial, yes, hacker, secure, local_firmware_server):
             helpers.from_websafe(json_content["firmware"]).encode()
         )
         crlf_firmware_content = b"\r\n".join(firmware_content.split(b"\n"))
-        # import IPython
-        # IPython.embed()
         m.update(crlf_firmware_content)
 
     our_digest = m.hexdigest()
